@@ -1,7 +1,7 @@
 #
 # The internetarchive module is a Python/CLI interface to Archive.org.
 #
-# Copyright (C) 2012-2021 Internet Archive
+# Copyright (C) 2012-2024 Internet Archive
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -20,7 +20,7 @@
 internetarchive.files
 ~~~~~~~~~~~~~~~~~~~~~
 
-:copyright: (C) 2012-2019 by Internet Archive.
+:copyright: (C) 2012-2024 by Internet Archive.
 :license: AGPL 3, see LICENSE for more details.
 """
 import logging
@@ -29,6 +29,7 @@ import socket
 import sys
 from contextlib import nullcontext, suppress
 from email.utils import parsedate_to_datetime
+from pathlib import Path
 from time import sleep
 from urllib.parse import quote
 
@@ -138,12 +139,25 @@ class File(BaseFile):
                 f'size={self.size!r}, '
                 f'format={self.format!r})')
 
-    def download(# noqa: max-complexity=38
-                 self, file_path=None, verbose=None, ignore_existing=None,
-                 checksum=None, destdir=None, retries=None, ignore_errors=None,
-                 fileobj=None, return_responses=None, no_change_timestamp=None,
-                 params=None, chunk_size=None, stdout=None, ors=None,
-                 timeout=None):
+    def download(  # noqa: C901,PLR0911,PLR0912,PLR0915
+        self,
+        file_path=None,
+        verbose=None,
+        ignore_existing=None,
+        checksum=None,
+        checksum_archive=None,
+        destdir=None,
+        retries=None,
+        ignore_errors=None,
+        fileobj=None,
+        return_responses=None,
+        no_change_timestamp=None,
+        params=None,
+        chunk_size=None,
+        stdout=None,
+        ors=None,
+        timeout=None,
+    ):
         """Download the file into the current working directory.
 
         :type file_path: str
@@ -159,6 +173,11 @@ class File(BaseFile):
         :type checksum: bool
         :param checksum: (optional) Skip downloading file based on checksum.
 
+        :type checksum_archive: bool
+        :param checksum_archive: (optional) Skip downloading file based on checksum, and
+                                 skip checksum validation if it already succeeded
+                                 (will create and use _checksum_archive.txt).
+
         :type destdir: str
         :param destdir: (optional) The directory to download files to.
 
@@ -201,6 +220,7 @@ class File(BaseFile):
         verbose = False if verbose is None else verbose
         ignore_existing = False if ignore_existing is None else ignore_existing
         checksum = False if checksum is None else checksum
+        checksum_archive = False if checksum_archive is None else checksum_archive
         retries = retries or 2
         ignore_errors = ignore_errors or False
         return_responses = return_responses or False
@@ -214,6 +234,14 @@ class File(BaseFile):
         self.item.session.mount_http_adapter(max_retries=retries)
         file_path = file_path or self.name
 
+        # Critical security check: Sanitize only the filename portion of file_path to
+        # prevent invalid characters and potential directory traversal issues.
+        # We use `utils.sanitize_filepath` instead of `utils.sanitize_filename` because:
+        # - `sanitize_filepath` preserves the directory path intact (does not encode path separators),
+        # - allowing `os.makedirs` to create intermediate directories correctly,
+        # - while still sanitizing just the filename to ensure it is safe for filesystem use.
+        file_path = utils.sanitize_filepath(file_path)
+
         if destdir:
             if return_responses is not True:
                 try:
@@ -224,8 +252,60 @@ class File(BaseFile):
                 raise OSError(f'{destdir} is not a directory!')
             file_path = os.path.join(destdir, file_path)
 
+        # Critical security check: Prevent directory traversal attacks by ensuring
+        # the download path doesn't escape the target directory using path resolution
+        # and relative path validation. This protects against malicious filenames
+        # containing ../ sequences or other path manipulation attempts.
+        try:
+            # Resolve both paths to handle symlinks and absolute paths
+            target_path = Path(file_path).resolve()
+            base_dir = Path(destdir).resolve() if destdir else Path.cwd().resolve()
+            # Ensure the target path is relative to base directory
+            target_path.relative_to(base_dir)
+        except ValueError:
+            raise ValueError(f"Download path {file_path} is outside target directory {base_dir}")
+
         parent_dir = os.path.dirname(file_path)
 
+        # Check if we should skip...
+        if not return_responses and os.path.exists(file_path.encode('utf-8')):
+            if checksum_archive:
+                checksum_archive_filename = '_checksum_archive.txt'
+                if not os.path.exists(checksum_archive_filename):
+                    with open(checksum_archive_filename, 'w', encoding='utf-8') as f:
+                        pass
+                with open(checksum_archive_filename, encoding='utf-8') as f:
+                    checksum_archive_data = f.read().splitlines()
+                if file_path in checksum_archive_data:
+                    msg = (
+                        f'skipping {file_path}, '
+                        f'file already exists based on checksum_archive.'
+                    )
+                    log.info(msg)
+                    if verbose:
+                        print(f' {msg}', file=sys.stderr)
+                    return
+            if ignore_existing:
+                msg = f'skipping {file_path}, file already exists.'
+                log.info(msg)
+                if verbose:
+                    print(f' {msg}', file=sys.stderr)
+                return
+            elif checksum or checksum_archive:
+                with open(file_path, 'rb') as fp:
+                    md5_sum = utils.get_md5(fp)
+
+                if md5_sum == self.md5:
+                    msg = f'skipping {file_path}, file already exists based on checksum.'
+                    log.info(msg)
+                    if verbose:
+                        print(f' {msg}', file=sys.stderr)
+                    if checksum_archive:
+                        # add file to checksum_archive to skip it next time
+                        with open(checksum_archive_filename, 'a', encoding='utf-8') as f:
+                            f.write(f'{file_path}\n')
+                    return
+
         # Retry loop
         while True:
             try:
@@ -237,15 +317,17 @@ class File(BaseFile):
                         and self.name != f'{self.identifier}_files.xml' \
                         and os.path.exists(file_path.encode('utf-8')):
                     st = os.stat(file_path.encode('utf-8'))
-                    if st.st_size != self.size and not checksum:
+                    if st.st_size != self.size and not (checksum or checksum_archive):
                         headers = {"Range": f"bytes={st.st_size}-"}
 
-                response = self.item.session.get(self.url,
-                                                 stream=True,
-                                                 timeout=timeout,
-                                                 auth=self.auth,
-                                                 params=params,
-                                                 headers=headers)
+                response = self.item.session.get(
+                    self.url,
+                    stream=True,
+                    timeout=timeout,
+                    auth=self.auth,
+                    params=params,
+                    headers=headers,
+                )
                 # Get timestamp from Last-Modified header
                 last_mod_header = response.headers.get('Last-Modified')
                 if last_mod_header:
@@ -256,35 +338,17 @@ class File(BaseFile):
 
                 response.raise_for_status()
 
-                # Check if we should skip...
-                if not return_responses and os.path.exists(file_path.encode('utf-8')):
-                    if ignore_existing:
-                        msg = f'skipping {file_path}, file already exists.'
-                        log.info(msg)
-                        if verbose:
-                            print(f' {msg}', file=sys.stderr)
-                        return
-                    elif checksum:
-                        with open(file_path, 'rb') as fp:
-                            md5_sum = utils.get_md5(fp)
-
-                        if md5_sum == self.md5:
-                            msg = f'skipping {file_path}, file already exists based on checksum.'
+                # Check if we should skip based on last modified time...
+                if not fileobj and not return_responses and os.path.exists(file_path.encode('utf-8')):
+                    st = os.stat(file_path.encode('utf-8'))
+                    if st.st_mtime == last_mod_mtime:
+                        if self.name == f'{self.identifier}_files.xml' or (st.st_size == self.size):
+                            msg = (f'skipping {file_path}, file already exists based on '
+                                    'length and date.')
                             log.info(msg)
                             if verbose:
                                 print(f' {msg}', file=sys.stderr)
                             return
-                    elif not fileobj:
-                        st = os.stat(file_path.encode('utf-8'))
-                        if st.st_mtime == last_mod_mtime:
-                            if self.name == f'{self.identifier}_files.xml' \
-                                or (st.st_size == self.size):
-                                msg = (f'skipping {file_path}, file already exists based on '
-                                        'length and date.')
-                                log.info(msg)
-                                if verbose:
-                                    print(f' {msg}', file=sys.stderr)
-                                return
 
                 elif return_responses:
                     return response
@@ -388,7 +452,7 @@ class File(BaseFile):
         :param verbose: (optional) Print actions to stdout.
 
         :type debug: bool
-        :param debug: (optional) Set to True to print headers to stdout and exit exit
+        :param debug: (optional) Set to True to print headers to stdout and exit
                       without sending the delete request.
 
         """
@@ -419,7 +483,7 @@ class File(BaseFile):
         else:
             if verbose:
                 msg = f' deleting: {self.name}'
-                if cascade_delete:
+                if cascade_delete == '1':
                     msg += ' and all derivative files.'
                 print(msg, file=sys.stderr)
             prepared_request = self.item.session.prepare_request(request)
