@@ -1,7 +1,7 @@
 #
 # The internetarchive module is a Python/CLI interface to Archive.org.
 #
-# Copyright (C) 2012-2021 Internet Archive
+# Copyright (C) 2012-2024 Internet Archive
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -20,7 +20,7 @@
 internetarchive.item
 ~~~~~~~~~~~~~~~~~~~~
 
-:copyright: (C) 2012-2021 by Internet Archive.
+:copyright: (C) 2012-2024 by Internet Archive.
 :license: AGPL 3, see LICENSE for more details.
 """
 from __future__ import annotations
@@ -34,7 +34,7 @@ from fnmatch import fnmatch
 from functools import total_ordering
 from logging import getLogger
 from time import sleep
-from typing import Mapping, MutableMapping
+from typing import Mapping, MutableMapping, Optional
 from urllib.parse import quote
 from xml.parsers.expat import ExpatError
 
@@ -493,6 +493,36 @@ class Item(BaseItem):
         r.raise_for_status()
         return r
 
+    def index_review(self, username=None, screenname=None, itemname=None) -> Response:
+        u = f'{self.session.protocol}//{self.session.host}/services/reviews.php'
+        p = {'identifier': self.identifier}
+        d = {'noindex': '0'}
+        if username:
+            d['username'] = username
+        elif screenname:
+            d['screenname'] = screenname
+        elif itemname:
+            d['itemname'] = itemname
+        a = S3Auth(self.session.access_key, self.session.secret_key)
+        r = self.session.put(u, params=p, data=d, auth=a)
+        r.raise_for_status()
+        return r
+
+    def noindex_review(self, username=None, screenname=None, itemname=None) -> Response:
+        u = f'{self.session.protocol}//{self.session.host}/services/reviews.php'
+        p = {'identifier': self.identifier}
+        d = {'noindex': '1'}
+        if username:
+            d['username'] = username
+        elif screenname:
+            d['screenname'] = screenname
+        elif itemname:
+            d['itemname'] = itemname
+        a = S3Auth(self.session.access_key, self.session.secret_key)
+        r = self.session.put(u, params=p, data=d, auth=a)
+        r.raise_for_status()
+        return r
+
     def delete_review(self, username=None, screenname=None, itemname=None) -> Response:
         u = f'{self.session.protocol}//{self.session.host}/services/reviews.php'
         p = {'identifier': self.identifier}
@@ -532,8 +562,8 @@ class Item(BaseItem):
     def get_files(self,
                   files: File | list[File] | None = None,
                   formats: str | list[str] | None = None,
-                  glob_pattern: str | None = None,
-                  exclude_pattern: str | None = None,
+                  glob_pattern: str | list[str] | None = None,
+                  exclude_pattern: str | list[str] | None = None,
                   on_the_fly: bool = False):
         files = files or []
         formats = formats or []
@@ -589,6 +619,7 @@ class Item(BaseItem):
                  verbose: bool = False,
                  ignore_existing: bool = False,
                  checksum: bool = False,
+                 checksum_archive: bool = False,
                  destdir: str | None = None,
                  no_directory: bool = False,
                  retries: int | None = None,
@@ -602,7 +633,7 @@ class Item(BaseItem):
                  exclude_source: str | list[str] | None = None,
                  stdout: bool = False,
                  params: Mapping | None = None,
-                 timeout: int | float | tuple[int, float] | None = None
+                 timeout: float | tuple[int, float] | None = None
                  ) -> list[Request | Response]:
         """Download files from an item.
 
@@ -627,6 +658,10 @@ class Item(BaseItem):
 
         :param checksum: Skip downloading file based on checksum.
 
+        :param checksum_archive: Skip downloading file based on checksum, and skip
+                                 checksum validation if it already succeeded
+                                 (will create and use _checksum_archive.txt).
+
         :param destdir: The directory to download files to.
 
         :param no_directory: Download files to current working
@@ -670,6 +705,7 @@ class Item(BaseItem):
         ignore_existing = bool(ignore_existing)
         ignore_errors = bool(ignore_errors)
         checksum = bool(checksum)
+        checksum_archive = bool(checksum_archive)
         no_directory = bool(no_directory)
         return_responses = bool(return_responses)
         no_change_timestamp = bool(no_change_timestamp)
@@ -746,8 +782,8 @@ class Item(BaseItem):
                 ors = True
             else:
                 ors = False
-            r = f.download(path, verbose, ignore_existing, checksum, destdir,
-                           retries, ignore_errors, fileobj, return_responses,
+            r = f.download(path, verbose, ignore_existing, checksum, checksum_archive,
+                           destdir, retries, ignore_errors, fileobj, return_responses,
                            no_change_timestamp, params, None, stdout, ors, timeout)
             if return_responses:
                 responses.append(r)
@@ -778,8 +814,9 @@ class Item(BaseItem):
                         secret_key: str | None = None,
                         debug: bool = False,
                         headers: Mapping | None = None,
+                        reduced_priority: bool = False,
                         request_kwargs: Mapping | None = None,
-                        timeout: int | float | None = None,
+                        timeout: float | None = None,
                         refresh: bool = True) -> Request | Response:
         """Modify the metadata of an existing item on Archive.org.
 
@@ -804,6 +841,12 @@ class Item(BaseItem):
 
         :param refresh: Refresh the item metadata after the request.
 
+        :param reduced_priority: Submit your task at a lower priority.
+                                 This option is helpful to get around rate-limiting.
+                                 Your task will more likely be accepted, but it might
+                                 not run for a long time. Note that you still may be
+                                 subject to rate-limiting.
+
         :returns: A Request if debug else a Response.
 
         Usage::
@@ -845,7 +888,8 @@ class Item(BaseItem):
             append=append,
             expect=expect,
             append_list=append_list,
-            insert=insert)
+            insert=insert,
+            reduced_priority=reduced_priority)
         # Must use Session.prepare_request to make sure session settings
         # are used on request!
         prepared_request = request.prepare()
@@ -857,6 +901,39 @@ class Item(BaseItem):
             self.refresh()
         return resp
 
+    def delete_flag(
+            self,
+            category: str,
+            user: Optional[str] = None,  # noqa: UP007
+    ) -> Response:
+        if user is None:
+            user = f"@{self.session.config.get('general', {}).get('screenname')}"
+        url = f'{self.session.protocol}//{self.session.host}/services/flags/admin.php'
+        headers = {'Accept': 'text/json'}  # must be text/json specifically
+        params = {'identifier': self.identifier, 'category': category, 'user': user}
+        r = self.session.delete(url, headers=headers, params=params)
+        return r
+
+    def add_flag(
+            self,
+            category: str,
+            user: Optional[str] = None,  # noqa: UP007
+    ) -> Response:
+        if user is None:
+            user = f"@{self.session.config.get('general', {}).get('screenname')}"
+        url = f'{self.session.protocol}//{self.session.host}/services/flags/admin.php'
+        headers = {'Accept': 'text/json'}  # must be text/json specifically
+        params = {'identifier': self.identifier, 'category': category, 'user': user}
+        r = self.session.put(url, headers=headers, params=params)
+        return r
+
+    def get_flags(self) -> Response:
+        url = f'{self.session.protocol}//{self.session.host}/services/flags/admin.php'
+        headers = {'Accept': 'text/json'}  # must be text/json specifically
+        params = {'identifier': self.identifier}
+        r = self.session.get(url, headers=headers, params=params)
+        return r
+
     # TODO: `list` parameter name shadows the Python builtin
     def remove_from_simplelist(self, parent, list) -> Response:
         """Remove item from a simplelist.
@@ -875,7 +952,7 @@ class Item(BaseItem):
         r = self.session.post(self.urls.metadata, data=data)  # type: ignore
         return r
 
-    def upload_file(self, body,
+    def upload_file(self, body,  # noqa: PLR0915; TODO: Refactor this method to reduce complexity
                     key: str | None = None,
                     metadata: Mapping | None = None,
                     file_metadata: Mapping | None = None,
@@ -891,8 +968,7 @@ class Item(BaseItem):
                     retries_sleep: int | None = None,
                     debug: bool = False,
                     validate_identifier: bool = False,
-                    request_kwargs: MutableMapping | None = None,
-                    set_scanner: bool = True) -> Request | Response:
+                    request_kwargs: MutableMapping | None = None) -> Request | Response:
         """Upload a single file to an item. The item will be created
         if it does not exist.
 
@@ -1055,8 +1131,7 @@ class Item(BaseItem):
                                 file_metadata=file_metadata,
                                 access_key=access_key,
                                 secret_key=secret_key,
-                                queue_derive=queue_derive,
-                                set_scanner=set_scanner)
+                                queue_derive=queue_derive)
             return request
 
         if debug:
@@ -1065,12 +1140,23 @@ class Item(BaseItem):
             return prepared_request
         else:
             try:
+                first_try = True
                 while True:
                     error_msg = ('s3 is overloaded, sleeping for '
                                  f'{retries_sleep} seconds and retrying. '
                                  f'{retries} retries left.')
-                    if retries > 0:
-                        if self.session.s3_is_overloaded(access_key=access_key):
+                    if retries > 0 and not first_try:
+                        try:
+                            overloaded = self.session.s3_is_overloaded(
+                                    access_key=access_key)
+                        except Exception as e:
+                            error_msg = ('error checking if s3 is overloaded via '
+                                         's3.us.archive.org?check_limit=1, '
+                                         f'exception raised: "{e}". '
+                                         f'sleeping for {retries_sleep} seconds and '
+                                         f'retrying. {retries} retries left.')
+                            overloaded = True
+                        if overloaded:
                             sleep(retries_sleep)
                             log.info(error_msg)
                             if verbose:
@@ -1098,6 +1184,7 @@ class Item(BaseItem):
                             print(f' warning: {error_msg}', file=sys.stderr)
                         sleep(retries_sleep)
                         retries -= 1
+                        first_try = False
                         continue
                     else:
                         if response.status_code == 503:
@@ -1146,8 +1233,7 @@ class Item(BaseItem):
                retries_sleep: int | None = None,
                debug: bool = False,
                validate_identifier: bool = False,
-               request_kwargs: dict | None = None,
-               set_scanner: bool = True) -> list[Request | Response]:
+               request_kwargs: dict | None = None) -> list[Request | Response]:
         r"""Upload files to an item. The item will be created if it
         does not exist.
 
@@ -1255,8 +1341,7 @@ class Item(BaseItem):
                                             retries_sleep=retries_sleep,
                                             debug=debug,
                                             validate_identifier=validate_identifier,
-                                            request_kwargs=request_kwargs,
-                                            set_scanner=set_scanner)
+                                            request_kwargs=request_kwargs)
                     responses.append(resp)
             else:
                 file_index += 1
@@ -1290,8 +1375,7 @@ class Item(BaseItem):
                                         retries_sleep=retries_sleep,
                                         debug=debug,
                                         validate_identifier=validate_identifier,
-                                        request_kwargs=request_kwargs,
-                                        set_scanner=set_scanner)
+                                        request_kwargs=request_kwargs)
                 responses.append(resp)
         return responses
 
